package edu.ntua.ivml.common.lp;

public class Predicate extends Functional implements Comparable<Predicate> {
	
	public static Predicate TRUTH_PREDICATE = new Predicate("t", 0);
	public static Predicate FALSE_PREDICATE = new Predicate("f", 0);
	
	public static String QUERY_PREDICATE_NAME = "Q";
	
	public Predicate(String name, int arity) {
		super(name, arity);
	}

	public boolean isConcept() {
		return arity == 1;
	}
	
	public boolean isRole() {
		return arity == 2;
	}
	
	public Atom toGeneralAtom() {
		Variable[] vars = new Variable[arity];
		
		for (int i = 0; i < vars.length; i++) {
			vars[i] = new Variable("x" + i);
		}
		
		return new Atom(this, vars);
	}
	
	public boolean equals(Object obj) {
		if (!(obj instanceof Predicate)) {
			return false;
		} else {
			return name.equals(((Predicate)obj).getName()) && arity == ((Predicate)obj).getArity();
		}
	}
	
	public int hashCode() {
		if (name != null) {
			return name.hashCode() + arity;
		} else {
			return 0;
		}
	}
	
	public int compareTo(Predicate p) {
		
		int nc = name.compareTo(p.getName());
		if (nc != 0) {
			return nc;
		} else {
			if (arity < p.getArity()) {
				return -1;
			} else if (arity > p.getArity()) {
				return 1;
			} else {
				return 0;
			}
		}
	}

	public boolean isQueryPredicate() {
		return name.equals(QUERY_PREDICATE_NAME);
	}
	
	public boolean isArithmeticPredicate() {
		return false;
	}

}