package edu.ntua.ivml.common.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

public class SetUtils {

	public static <U> boolean areNullOrEqual(Collection<U> s1, Collection<U> s2) {
		
		if (s1 == null || s2 == null) {
			return true;
		} 
		
		for (U o : s1) {
			if (!s2.contains(o)) {
				return false;
			}
		}

		for (U o : s2) {
			if (!s1.contains(o)) {
				return false;
			}
		}
		
		return true;
	}			
	
	public static <U> boolean areDisjoint(Set<U> s1, Set<U> s2) {
		for (U o : s2) {
			if (s1.contains(o)) {
				return false;
			}
		}
		
		return true;
	}

	public static <U> Set<Set<U>> mergeSets(Set<Set<U>> sets, Set<U> newSet) {
		Set<Set<U>> res = new HashSet<Set<U>>();
		
		Set<U> mergedSet = new HashSet<U>();

		for (Set<U> eSet : sets) {
			if (!SetUtils.areDisjoint(eSet, newSet)) {
				mergedSet.addAll(eSet);
			} else {
				res.add(eSet);
			}
		}

		mergedSet.addAll(newSet);
		
		res.add(mergedSet);
		
		return res;
	}
	
	public static <U> Set<U> elementDifference(Collection<U> s1, Collection<U> s2) {
		
		Set<U> res = new HashSet<U>();
		
		if (s1 == null && s2 == null) {
			return res;
		} else if (s1 == null) {
			res.addAll(s2);
			return res;
		} else if (s2 == null) {
			res.addAll(s1);
			return res;
		}
		
		for (U o : s1) {
			if (!s2.contains(o)) {
				res.add(o);
			}
		}

		for (U o : s2) {
			if (!s1.contains(o)) {
				res.add(o);
			}
		}
		
		return res;
	}	
	
	public static <U> boolean areEqual(Collection<U> s1, Collection<U> s2) {
		
		if (s1 == null && s2 == null) {
			return true;
		} else if (s1 == null || s2 == null) {
			return false;
		} 
		
		for (Object o : s1) {
			if (!s2.contains(o)) {
				return false;
			}
		}

		for (Object o : s2) {
			if (!s1.contains(o)) {
				return false;
			}
		}
		
		return true;
	}	
	
	public static <U> Set<U> intersection(Collection<U> s1, Collection<U> s2) {
		
		Set<U> res = new HashSet<U>();
		
		for (U o : s1) {
			if (s2.contains(o)) {
				res.add(o);
			}
		}
		return res;
	}	

	public static <U> Set<U> union(Collection<U>... sets) {
		
		Set<U> res = new HashSet<U>();
		
		for (Collection<U> s : sets) {
			res.addAll(s);
		}

		return res;
	}
	
	public static <U> Set<U> difference(Collection<U> s1, Collection<U> s2) {
		
		Set<U> res = new HashSet<U>();
		
		for (U o : s1) {
			if (!s2.contains(o)) {
				res.add(o);
			}
		}
		
		return res;
	}		
	
	public static Set flatten(Set<?> s) {
		
		Set res = new HashSet();
		
		for (Object o : s) {
			if (o instanceof Set) {
				res.addAll(flatten((Set)o));
			} else {
				res.add(o);
			}
		}
		
		return res;
		
	}
	
	public static <T> Set<T> objectToSet(T obj) {
		Set<T> set = new HashSet<T>();
		set.add(obj);
		
		return set;
	}

	public static <T> ArrayList<Set<T>> cloneArrayListSet(ArrayList<Set<T>> cover) {
		ArrayList<Set<T>> res = new ArrayList<Set<T>>();
		
		for (Set<T> s : cover) {
			res.add((Set)((HashSet)s).clone());
		}
		
		return res;
	}



}
