package edu.ntua.ivml.qa.algorithm.rapid.dllite;

import java.util.HashSet;
import java.util.Set;

import edu.ntua.ivml.common.lp.Atom;
import edu.ntua.ivml.common.lp.Variable;
import edu.ntua.ivml.qa.algorithm.rapid.ClauseUnfolder;
import edu.ntua.ivml.qa.algorithm.rapid.UnfoldedClause;


public class DLFastSubsumeChecker extends DLSubsumeChecker {
	
	public UnfoldedClause check(Atom head, DLIndexEntryList[] atomList, int[] index, boolean isSymmetric, ClauseUnfolder unf) {
		boolean pure = !isSymmetric;
		
		UnfoldedClause newClause = null;
		
		Set<DLIndexEntry> aies = new HashSet<DLIndexEntry>();
		Set<Set<DLIndexEntry>> roots = new HashSet<Set<DLIndexEntry>>();
		Set<DLIndexEntry> haveChildren = new HashSet<DLIndexEntry>();

		for (int j = 0; j < atomList.length; j++) {
			
			if (index[j] != -1) {
				DLIndexEntry aie = (DLIndexEntry)atomList[j].getList().get(index[j]);
				aies.add(aie);
				
				if (aie.roots != null) {
					DLIndexEntry unboundRoot = null; 

					for (DLIndexEntry ie : aie.roots) {
						boolean allUnbound = true;
						for (Variable v : ie.getAtom().getVariables()) {
							if (!v.hasUnboundPrefix()) {
								allUnbound = false;
								break;
							}
						}
						
						if (allUnbound) {
							unboundRoot = ie;
						}
					}
					
					if (unboundRoot == null) {
						roots.add(aie.roots);
					} else {
						Set<DLIndexEntry> ur = new HashSet<DLIndexEntry>();
						ur.add(unboundRoot);
						
						roots.add(ur);
						
						Set<DLIndexEntry> rootsCopy = (Set<DLIndexEntry>)((HashSet)aie.roots).clone();
						rootsCopy.remove(unboundRoot);
						
						if (!rootsCopy.isEmpty()) {
							roots.add(rootsCopy);
						}
					}
				}
				
				if (aie.children != null) {
					haveChildren.add(aie);
				}
			}
			
		}

		if (!roots.isEmpty()) {
			Set<DLIndexEntry>[] rootsArray = roots.toArray(new Set[] {});
			
			for (int i = 0; i < rootsArray.length; i++) {
				Set<DLIndexEntry> tmp = new HashSet<DLIndexEntry>();
				tmp.addAll(rootsArray[i]);

				Set<DLIndexEntry> ch = new HashSet<DLIndexEntry>(); 
				for (DLIndexEntry r : rootsArray[i]) {
					ch.addAll(r.children);
				}

				for (DLIndexEntry a : aies) {
					if (!ch.contains(a)) {
						tmp.add(a);
					}
				}

				Set<Integer> covered = new HashSet<Integer>();
				for (DLIndexEntry a : tmp) {
					covered.addAll(a.covered);
				}

				if (covered.size() == atomList.length) {
					return null;
				}
			}
		}
		
		if (newClause == null) {
			newClause = unf.createClause(adjustIndex(index), pure);
		}
		
		return newClause;
	}
}