package edu.ntua.ivml.qa.algorithm.rapid.dllite;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import edu.ntua.ivml.common.lp.Atom;
import edu.ntua.ivml.common.lp.Clause;
import edu.ntua.ivml.common.lp.Term;
import edu.ntua.ivml.qa.algorithm.rapid.DatalogAtom;
import edu.ntua.ivml.qa.algorithm.rapid.MarkedClause;
import edu.ntua.ivml.qa.algorithm.rapid.Rapid;


public class DLRapid extends Rapid {

	private int mode;
	
	public static DLRapid createDatalogRapid() {
		return new DLRapid(1, false);
	}

	public static DLRapid createSimpleUnfoldRapid() {
		return new DLRapid(1, true);
	}

	public static DLRapid createFastUnfoldRapid() {
		return new DLRapid(2, true);
	}

	private DLRapid(int mode, boolean unfold) {
		super("DLRapid-" + mode + (unfold == true ? " unfold":""), unfold);
		
		this.mode = mode;
	}
	
	public boolean isELHI() {
		return false;
	}
	
	public void initialize() {
		flattener = new DLFlattener();
		atomUnfolder = new DLAtomUnfolder(lp);
		
		if (mode == 1) {
			clauseUnfolder = new DLUnfolder(lp, atomUnfolder, new DLSimpleSubsumeChecker());
		} else if (mode == 2) {
			clauseUnfolder = new DLUnfolder(lp, atomUnfolder, new DLFastSubsumeChecker());
		}
	}
	
	public ArrayList<Clause> postsc(ArrayList<Clause> c) {
		return c;
	}
	
	protected ArrayList<Clause> strip(ArrayList<MarkedClause> queryList) {
		ArrayList<Clause> queries = new ArrayList<Clause>();
	
		for (int k = 0; k < queryList.size(); k++) {
			queries.add(((DLUnfolder)clauseUnfolder).testUnfold(queryList.get(k).getClause(), null));
		}

		queries = Clause.getNonRedundantClauses(queries);

		return queries;
	}

}
