package edu.ntua.ivml.qa.algorithm.rapid.elhi;

import java.util.Map;

import edu.ntua.ivml.common.lp.Atom;
import edu.ntua.ivml.common.lp.Substitution;
import edu.ntua.ivml.common.lp.Term;
import edu.ntua.ivml.common.lp.Variable;
import edu.ntua.ivml.qa.algorithm.rapid.AtomUnfolding;
import edu.ntua.ivml.qa.algorithm.rapid.Flattener;
import edu.ntua.ivml.qa.algorithm.rapid.FlattenerExtra;
import edu.ntua.ivml.qa.algorithm.rapid.Rapid;
import edu.ntua.ivml.qa.lp.theory.FCTheory;


public abstract class ETFlattener extends Flattener {

	protected FCTheory lp;
	
	public abstract AtomUnfolding createAtomUnfolding(VUnfoldStructure vus, Map<Atom, VUnfoldStructure> map, Atom head, FlattenerExtra extra);
	
	protected ETFlattener(FCTheory lp) {
		this.lp = lp;
	}

	
	protected Substitution appendSuffix(Substitution subst, int s) {
		Substitution res = new Substitution(); 
		
		for (Variable v : subst.getVariables()) {
			Variable t = (Variable)subst.get(v);
		
			if (!Rapid.isCoreVariable(t)) {
				res.put(v, new Variable(t.getName() + s));
			} else {
				res.put(v, t);
			}
		}
		
		return res;
	}
	
	public static boolean formEquals(Atom a1, Atom a2) {
		if (!a1.getPredicate().equals(a2.getPredicate())) {
			return false;
		}
		
		Term[] t1 = a1.getArguments().getTerms();
		Term[] t2 = a2.getArguments().getTerms();
		
		for (int i = 0; i < t1.length; i++) {
			if (t1[i].isConstant() && t1[i].equals(t2[i])) {
				continue;
			} else if (t1[i].isVariable() && t2[i].isVariable()) {
				Variable v1 = (Variable)t1[i];
				Variable v2 = (Variable)t2[i];
				
				if (v1.hasUnboundPrefix() && v2.hasUnboundPrefix()) {
					continue;
				} else if (!v1.hasUnboundPrefix() && !v2.hasUnboundPrefix()) {
					continue;
				}
			}
			
			return false;
		}
		
		return true;
	}
	
}
