package edu.ntua.ivml.qa.algorithm.rapid.elhi;

import java.util.ArrayList;

import edu.ntua.ivml.common.lp.Clause;
import edu.ntua.ivml.qa.algorithm.rapid.MarkedClause;
import edu.ntua.ivml.qa.algorithm.rapid.Rapid;

public class ETRapid extends Rapid {

	public boolean datalog;
	public boolean fdatalog;
	
	public static ETRapid createDatalogRapid() {
		return new ETRapid(true, true);
	}

	protected ETRapid(boolean datalog, boolean fdatalog) {
		super("ETRapid" + (fdatalog ? "-datalog" : (datalog ? "-unfold" : "-expand")), fdatalog ? false : true);
		this.datalog = datalog;
		this.fdatalog = fdatalog;
	}
	
	public boolean isELHI() {
		return true;
	}
	
	public void execute() {
		if (datalog) {
			qc.setRemoveAUX(false);
		}
		super.execute();
	}
	
	public ArrayList<Clause> postsc(ArrayList<Clause> clauses) {
		ArrayList<Clause> rClauses = clauses; 
		if (!datalog) {
			rClauses = Clause.getNonRedundantClauses(clauses);
		}
		
		return super.postsc(rClauses);
	}

	public void initialize() {
		if (fdatalog) {
			flattener = new ETDatalogFlattener(lp);
			atomUnfolder = new ETAtomUnfolder(lp, (ETFlattener)flattener);
			clauseUnfolder = null;
		}
	}
	
	protected ArrayList<Clause> strip(ArrayList<MarkedClause> queryList) {
		ArrayList<Clause> res = new ArrayList<Clause>();
		ArrayList<Integer> ids = new ArrayList<Integer>();
		
		int id = 0;
		int pid = 0;
		for (int k = 0; k < queryList.size(); k++) {
			res.add(queryList.get(k).getClause());
			
			if (k == 0) {
				id = 0;
				pid = queryList.get(k).getMark();
			} else {
				int kid = queryList.get(k).getMark();
				if (kid != pid) {
					id++;
					pid = kid;
				}
			}
			
			ids.add(id);
		}

		ArrayList<Clause> ret = Clause.getNonRedundantClauses(res, ids);

		
		return ret;
	}

}
